#(c)www.stani.be (read __doc__ for more information)
import sm
INFO=sm.INFO.copy()

INFO['description']=\
"""General Python scripts."""

__doc__=INFO['doc']%INFO
#_______________________________________________________________________________

####IMPORT----------------------------------------------------------------------
import math,time,string

####CLASSES---------------------------------------------------------------------

class New:
    """Empty class of which properties can be set through keywords."""
    def __init__(self,**keywords):
        self.__dict__=keywords

#---Stunt Class-----------------------------------------------------------------
APPEND='__www.stani.be__'#just an unique value
APPEND_METHODS=[]

class _StuntControlMethod:
    """Call with arguments and keywords.

    See Stunt class for more information.
    """
    def __init__(self,control,method,appendMethods=APPEND_METHODS):
        self.method=method
        self.appendMethods=appendMethods
        self.argKey=[]
    def __call__(self,*arguments,**keywords):
        try:
            appendArgument=(APPEND == arguments[-1])
        except:
            appendArgument=0
        if (self.method in self.appendMethods) or appendArgument:
            if appendArgument:
                arguments=arguments[:-1]
            self.argKey.append((arguments,keywords))
        else:self.argKey=[(arguments,keywords)]

class _StuntControl:
    """Buffer to register all calls to dialog control methods. Usefull for threads.

    See Stunt class for more information.
    """
    def __init__(self,control):
        self.control=control
        self.methods={}
    def __getattr__(self,method):
        if public(method):
            if method not in self.methods.keys():
                self.methods[method]= _StuntControlMethod(self,method)
            return self.methods[method]
        else:return self.__dict__[method]
    def __call__(self,containerControl):
        for method,StuntMethod in self.methods.items():
            for arguments,keywords in StuntMethod.argKey:apply(getattr(containerControl,method),arguments,keywords)
        self.methods.clear()

class Stunt:
    """Buffer to register all calls to a dialog. Usefull for threads.

    These actions can be applied later through for example a dialog timer event.
    Example:
        >>> dialog=Stunt()
        >>> dialog.gauge.SetRange(100)
        >>> dialog.gauge.SetValue(25)
        >>> dialog.label.SetValue('Hello world')
        >>> dialog.controls
        {'gauge': <smNew._StuntControl instance at 0x0165BA18>, 'label': <smNew._StuntControl instance at 0x0165B980>}
        >>> dialog.controls['gauge'].methods
        {'SetValue': <smNew._StuntControlMethod instance at 0x0166FAF8>, 'SetRange': <smNew._StuntControlMethod instance at 0x0166F940>}
        >>> dialog.controls['gauge'].methods['SetValue'].argKey
        [((25,), {})]
        #>> dialog(wxDialog)
    """
    def __init__(self,container=None):
        self.controls={}
        self.__container=container
    def __getattr__(self,control):
        if public(control):
            if control not in self.controls.keys():
                self.controls[control]= _StuntControl(control)
            return self.controls[control]
        else:return self.__dict__[control]
    def __call__(self):
        "Apply Stunted methods of self to container."
        self.busy=1
        for control,StuntControl in self.controls.items():StuntControl(getattr(self.__container,control))
        self.controls.clear()
        self.busy=0

####FUNCTIONS-------------------------------------------------------------------

def assertList(x):
    "Force x to a list."
    if type(x).__name__=='list': return x
    else: return [x]

def distance(p1,p2):
    "Calculates distance between two 2d points."
    px=math.fabs(p1[0]-p2[0])+1
    py=math.fabs(p1[1]-p2[1])+1
    return math.sqrt(px*px+py*py)

def flatten(s):
    "Flattens a list."
    result = []
    for i in s:
        try:
            result = result + flatten(i)
        except TypeError:
            result.append(i)
    return result

def minSec(seconds):
    "Converts seconds to minutes:seconds string."
    seconds=int(seconds)
    return str(seconds/60)+':'+str(seconds%60)

def public(x):
    "Returns true if string x doesn't start with '__'."
    return x[:2]!='__'

def rstrip(x,char=' '):
    try:
        return x.rstrip('-')
    except:
        index=len(x)
        try:
            while x>0 and x[index-1]==char:
                index-=1
            return x[:index]
        except:
            return x

def strFill(s,n):
    "Fills a string with n times the substring s."
    return ''.ljust(n).replace(' ',s)

def subtract(l,m):
    "Subtract list m from list l"
    def _notCommon(x):
        return not(x in m)
    return filter(_notCommon,l)

def timePassed(x):
    "Returns the time as a string in min and sec since x."
    return minSec(time.time()-x)

def unique(s):
    """Return a list of the elements in s, but without duplicates.

    For example, unique([1,2,3,1,2,3]) is some permutation of [1,2,3],
    unique("abcabc") some permutation of ["a", "b", "c"], and
    unique(([1, 2], [2, 3], [1, 2])) some permutation of
    [[2, 3], [1, 2]].

    For best speed, all sequence elements should be hashable.  Then
    unique() will usually work in linear time.

    If not possible, the sequence elements should enjoy a total
    ordering, and if list(s).sort() doesn't raise TypeError it's
    assumed that they do enjoy a total ordering.  Then unique() will
    usually work in O(N*log2(N)) time.

    If that's not possible either, the sequence elements must support
    equality-testing.  Then unique() will usually work in quadratic
    time.
    """

    n = len(s)
    if n == 0:
        return []

    # Try using a dict first, as that's the fastest and will usually
    # work.  If it doesn't work, it will usually fail quickly, so it
    # usually doesn't cost much to *try* it.  It requires that all the
    # sequence elements be hashable, and support equality comparison.
    u = {}
    try:
        for x in s:
            u[x] = 1
    except TypeError:
        del u  # move on to the next method
    else:
        return u.keys()

    # We can't hash all the elements.  Second fastest is to sort,
    # which brings the equal elements together; then duplicates are
    # easy to weed out in a single pass.
    # NOTE:  Python's list.sort() was designed to be efficient in the
    # presence of many duplicate elements.  This isn't true of all
    # sort functions in all languages or libraries, so this approach
    # is more effective in Python than it may be elsewhere.
    try:
        t = list(s)
        t.sort()
    except TypeError:
        del t  # move on to the next method
    else:
        assert n > 0
        last = t[0]
        lasti = i = 1
        while i < n:
            if t[i] != last:
                t[lasti] = last = t[i]
                lasti = lasti+1
            i = i+1
        return t[:lasti]

    # Brute force is all that's left.
    u = []
    for x in s:
        if x not in u:
            u.append(x)
    return u

def zfill(x,width):
    try:
        return string.zfill(x,width)
    except:
        x=str(x)
        l=len(x)
        if width>l:
            return ('%0'+str(width-l)+'d%s')%(0,x)
        else:
            return x

####CONSTANTS-------------------------------------------------------------------
INCH2CM=2.54
CM2INCH=1/INCH2CM
MM2INCH=1/(INCH2CM*10)
MONTHS=('january','february','march','april','may','june','july','august','september','october','november','december')